/***************************************************************************
 *   Copyright (C) 2004 by Johan Maes - ON4QZ                              *
 *   on4qz@telenet.be                                                      *
 *   http://users.telenet.be/on4qz                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "bwmode.h"
#include "configdialog.h"
#include "dspfunctions.h"
#include "imageview.h"
#include "qsstvglobal.h"
#include <debugplot.h>



bwMode::bwMode(QObject *parent, const char *name)
 : modeBase(parent, name)
{
}

bwMode::~bwMode()
{
	
}

void bwMode::init(sstvMode imode,imageView *ptr,bool tx,bool initClock)
{
	modeBase::init(imode,ptr,tx,initClock);
	visibleLineLength=(((float)(lineTimeTable[9]))/10.-fp-bp-syncDuration);
}


/*! \fn bwMode::setupLine(int &subln,unsigned char **buf)
		\brief Decode a Black&White mode sstv image
	
		\param subln every line to display is divided in a number of sublines
		\param buf pointer to the pixel array.
	 	\return next phase off the display
*/ 


embState bwMode::setupLine(int &subln,unsigned char **buf)
{
	switch(subln)
		{
			case 0:
				calcPixelPositionTable(GREENLINE,FALSE);
				*buf=greenArray;
//				debug("setup green samplecount=%d",sampleCounter);
				return PIXELS;
			case 1:
				debugPtr->state=stSync;
				syncPosition=(unsigned int)round(lineTimeTable[lineCounter]);
				return SYNC;
			break;
			default:
				return ENDOFLINE;
		}
}


bool bwMode::showLine()
{
	unsigned int i;
	unsigned int *pixelArray=displayPtr->getLineAddress(lineCounter);
	for(i=0;i<activeSSTVParam.numberOfPixels;i++)
		{
			pixelArray[i]=qRgb(greenArray[i],greenArray[i],greenArray[i]);
		}
	displayPtr->showLine();
	lineCounter++;
	if (lineCounter>=activeSSTVParam.numberOfLines) return TRUE;
	return FALSE;
}

void bwMode::calcPixelPositionTable(unsigned int colorLine,bool tx)
{
	unsigned int i;
	int ofx=0;
	if(tx) ofx=1;
	double start=0;
	if(lineCounter!=0) start=lineTimeTable[lineCounter-1];
	debugPtr->state=colorLine;
	start+=0;
	for(i=0;i<activeSSTVParam.numberOfPixels;i++)
		{
			pixelPositionTable[i]=(unsigned int)round(start+(((float)(i+ofx)*visibleLineLength)/activeSSTVParam.numberOfPixels));
		}
}

embState bwMode::txSetupLine(int &subln,unsigned char **buf)
{
//	debug("startImageTX");
	switch(subln)
		{
			case 0:
				calcPixelPositionTable(GREENLINE,FALSE);
				*buf=greenArray;
				return PIXELS;
			case 1:
				txFreq=1500;
				txDur=fp;
				return TXGAP;
			case 2:
				txFreq=1200;
				txDur=syncDuration;
				return TXGAP;
			case 3:
				txFreq=1500;
				txDur=bp;
				return TXGAP;
			default:
				return ENDOFLINE;
		}
}

void bwMode::getLine()
{
	getLineBW();
}
