/***************************************************************************
 *   Copyright (C) 2004 by Johan Maes - ON4QZ                              *
 *   on4qz@telenet.be                                                      *
 *   http://users.telenet.be/on4qz                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "calibmode.h"
#include "configdialog.h"
#include "imageview.h"
#include "qsstvglobal.h"
#include <debugplot.h>




calibMode::calibMode(QObject *parent, const char *name)  : modeBase(parent, name)
{

}


calibMode::~calibMode()
{

}

void calibMode::init (sstvMode imode,imageView *p,bool tx)
{
	modeBase::init(imode,p,tx,TRUE);
	for(unsigned int i=0;i<activeSSTVParam.numberOfPixels;i++)
		{
			redArray[i]=0;
			blueArray[i]=0;
		}
	averageFreq=0;
	visibleLineLength=lineTimeTable[0];
}

/** \fn calibMode::setupLine(int &subln,unsigned char **buf)
		\brief Show calibration mode display
		
		The buffers are setup and the mode is returned
		\return mode for next step
	
*/


embState calibMode::setupLine(int &subln,unsigned char **buf)
{
	switch(subln)
		{
			case 0:
				calcPixelPositionTable(GREENLINE,FALSE);
				*buf=greenArray;
//				debug("setup green samplecount=%d",sampleCounter);
				return PIXELS;
			default:
				return ENDOFLINE;
		}
}


void calibMode::calcPixelPositionTable(unsigned int colorLine,bool)
{
	unsigned int i;
	double start=0;
	if(lineCounter!=0) start=lineTimeTable[lineCounter-1];
	debugPtr->state=colorLine;
	for(i=0;i<activeSSTVParam.numberOfPixels;i++)
		{
			
			pixelPositionTable[i]=(unsigned int)round(start+(((float)i*visibleLineLength)/activeSSTVParam.numberOfPixels));
		}
}

#define INTEGRA 0.4
bool calibMode::getPixels(unsigned char *pixelArray)
{
	float color=255;
	averageFreq=INTEGRA*averageFreq +(1.-INTEGRA)*sample;
	if(sampleCounter>=pixelPositionTable[pixelCounter])
		{
//			if((averageFreq>1100) && (averageFreq<1300)) color=0;
			if (sync) color=0;
			pixelArray[pixelCounter]=(unsigned char)color;
			pixelCounter++;
			if(pixelCounter>=activeSSTVParam.numberOfPixels) return TRUE;
		}
	return FALSE; // indicate, it's not the end of the line
}




embState calibMode::txSetupLine(int &,unsigned char **)
{
	return ENDOFLINE;
}

