/***************************************************************************
 *   Copyright (C) 2004 by Johan Maes - ON4QZ                              *
 *   on4qz@telenet.be                                                      *
 *   http://users.telenet.be/on4qz                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "galerymainwindow.h"
#include "galeryform.h"
#include <qsettings.h>
#include "qsstvglobal.h"
#include "utils.h"
#include <qstatusbar.h>
#include "thumbimage.h"
#include "imageview.h"


/*! \fn galeryMainWindow::galeryMainWindow()
		\brief constructor
*/
galeryMainWindow::galeryMainWindow()
 : QMainWindow()
{
	initView();
	readSettings();
	initActions();
	initToolbar();
	initMenubar();
	statusBar()->message("Starting ..");
}

/*! \fn galeryMainWindow::~galeryMainWindow()
		\brief destructor (saves settings on deletion)
		
		
*/
galeryMainWindow::~galeryMainWindow()
{
	writeSettings();
	
}



void galeryMainWindow::init()
{
}

void galeryMainWindow::initActions()
{
}
void galeryMainWindow::initToolbar()
{
}

void galeryMainWindow::initMenubar()
{
}

void galeryMainWindow::readSettings()
{
	int i;
	QSettings qSettings(QSettings::Ini);
	qSettings.beginGroup(APP_KEY+"/GAL/");
	int windowWidth = qSettings.readNumEntry( "galWindowWidth", 460 );
	int windowHeight = qSettings.readNumEntry("galWindowHeight", 530 );
	int windowX = qSettings.readNumEntry("galWindowX", -1 );
	int windowY = qSettings.readNumEntry("galWindowY", -1 );
	resize( windowWidth, windowHeight );
	rxIndex=qSettings.readNumEntry("rxIndex",0 );
	if ( windowX != -1 || windowY != -1 ) 	move( windowX, windowY );
	qSettings.endGroup();
	setParams();
	for(i=0;i<NUMTHUMBS;i++)
		{
			rxThumbsArray[i]->readSettings(&qSettings);
			txThumbsArray[i]->readSettings(&qSettings);
			templateThumbsArray[i]->readSettings(&qSettings);
		}
}

void galeryMainWindow::writeSettings()
{
	int i;
	QSettings qSettings(QSettings::Ini);
	getParams();
	qSettings.beginGroup(APP_KEY+"/GAL/");
	qSettings.writeEntry("galWindowWidth", width() );
	qSettings.writeEntry("galWindowHeight", height() );
  qSettings.writeEntry("galWindowX", x() );
  qSettings.writeEntry("galWindowY", y() );
	qSettings.writeEntry("rxIndex",rxIndex);
	qSettings.endGroup();
	for(i=0;i<NUMTHUMBS;i++)
		{
			rxThumbsArray[i]->writeSettings(&qSettings);
			txThumbsArray[i]->writeSettings(&qSettings);
			templateThumbsArray[i]->writeSettings(&qSettings);
		} 
	
}

void galeryMainWindow::initView()
{
	int i;
	gf= new galeryForm(this);
	setCentralWidget(gf);
	rxThumbsArray[0]= gf->rxThumbImage1;
	rxThumbsArray[1]= gf->rxThumbImage2;
	rxThumbsArray[2]= gf->rxThumbImage3;
	rxThumbsArray[3]= gf->rxThumbImage4;
	rxThumbsArray[4]= gf->rxThumbImage5;
	rxThumbsArray[5]= gf->rxThumbImage6;
	rxThumbsArray[6]= gf->rxThumbImage7;
	rxThumbsArray[7]= gf->rxThumbImage8;
	rxThumbsArray[8]= gf->rxThumbImage9;
	rxThumbsArray[9]= gf->rxThumbImage10;
	rxThumbsArray[10]=gf->rxThumbImage11;
	rxThumbsArray[11]=gf->rxThumbImage12;

	txThumbsArray[0]= gf->txThumbImage1;
	txThumbsArray[1]= gf->txThumbImage2;
	txThumbsArray[2]= gf->txThumbImage3;
	txThumbsArray[3]= gf->txThumbImage4;
	txThumbsArray[4]= gf->txThumbImage5;
	txThumbsArray[5]= gf->txThumbImage6;
	txThumbsArray[6]= gf->txThumbImage7;
	txThumbsArray[7]= gf->txThumbImage8;
	txThumbsArray[8]= gf->txThumbImage9;
	txThumbsArray[9]= gf->txThumbImage10;
	txThumbsArray[10]=gf->txThumbImage11;
	txThumbsArray[11]=gf->txThumbImage12;
	
	templateThumbsArray[0]= gf->templateThumbImage1;
	templateThumbsArray[1]= gf->templateThumbImage2;
	templateThumbsArray[2]= gf->templateThumbImage3;
	templateThumbsArray[3]= gf->templateThumbImage4;
	templateThumbsArray[4]= gf->templateThumbImage5;
	templateThumbsArray[5]= gf->templateThumbImage6;
	templateThumbsArray[6]= gf->templateThumbImage7;
	templateThumbsArray[7]= gf->templateThumbImage8;
	templateThumbsArray[8]= gf->templateThumbImage9;
	templateThumbsArray[9]= gf->templateThumbImage10;
	templateThumbsArray[10]=gf->templateThumbImage11;
	templateThumbsArray[11]=gf->templateThumbImage12;
	for(i=0;i<NUMTHUMBS;i++)
		{
			rxThumbsArray[i]->setType(thumbImage::RXTHUMB);
			txThumbsArray[i]->setType(thumbImage::TXTHUMB);
			templateThumbsArray[i]->setType(thumbImage::TEMPLATETHUMB);
		}
}

void galeryMainWindow::getParams()
{
	
}

void galeryMainWindow::setParams()
{

}

QString galeryMainWindow::getTemplateFileName(int tm)
{
	if(tm<NUMTHUMBS)
		{
			return templateThumbsArray[tm]->getFilename();
		}
	return "";
}

void galeryMainWindow::putRxImage(QString fn)
{
	rxThumbsArray[rxIndex]->setFilename(fn,"");
	rxThumbsArray[rxIndex]->load();
	rxIndex=(++rxIndex)%NUMTHUMBS;
}
