/***************************************************************************
 *   Copyright (C) 2004 by Johan Maes - ON4QZ                              *
 *   on4qz@telenet.be                                                      *
 *   http://users.telenet.be/on4qz                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gbr2mode.h"
#include "configdialog.h"
#include "dspfunctions.h"
#include "imageview.h"
#include "qsstvglobal.h"
#include <debugplot.h>


gbr2Mode::gbr2Mode(QObject *parent, const char *name)
 : modeBase(parent, name)
{
}


gbr2Mode::~gbr2Mode()
{
	
}


/*! \fn gbr2Mode::setupLine(int &subln,unsigned char **buf)
		\brief Decode a Scottie mode sstv image
	
		\param subln every line to display is divided in a number of sublines
		\param buf pointer to the pixel array.
	 	\return next phase off the display
*/ 

embState gbr2Mode::setupLine(int &subln,unsigned char **buf)
{
	// if we start from a viscode, then the first subline is a GREEN-subline
	// if we start from a sync then the we have to go to case 3: RED-subline
	
	double start=0;
	if(lineCounter!=0) start=lineTimeTable[lineCounter-1];
	switch(subln)
		{

			case 0:
				calcPixelPositionTable(GREENLINE,FALSE);
				*buf=greenArray;
//				debug("setup green samplecount=%d, pos=%d",sampleCounter,debugPtr->getPos());
				return PIXELS;
			case 1:
				debugPtr->state=stG1;
				marker=(unsigned int)round(start+blank+visibleLineLength);
				return RXWAIT;
			case 2:
				calcPixelPositionTable(BLUELINE,FALSE);
				*buf=blueArray;
//				debug("setup blue samplecount=%d,pos=%d",sampleCounter,debugPtr->getPos());
				return PIXELS;
			case 3:
				debugPtr->state=stFP;
				marker=(unsigned int)round(start+blank+fp+2*visibleLineLength);
				return RXWAIT;
			case 4:
				debugPtr->state=stSync;
				syncPosition=syncEndPosition;
//			debug("samplecount=%d,synPos=%d,pos=%d",sampleCounter,syncPosition,debugPtr->getPos());
				return SYNC;
			case 5:
				debugPtr->state=stBP;
				marker=(unsigned int)round(syncEndPosition+bp);
				return RXWAIT;
			case 6:
				calcPixelPositionTable(REDLINE,FALSE);
//				debug("setup red samplecount=%d,pos=%d",sampleCounter,debugPtr->getPos());
				*buf=redArray;
				return PIXELS;
			case 7:
				debugPtr->state=stG1;
				marker=(unsigned int)round(lineTimeTable[lineCounter]);
				return RXWAIT;
			break;
			default:
				return ENDOFLINE;
		}
}


void gbr2Mode::calcPixelPositionTable(unsigned int colorLine,bool tx)
{
	unsigned int i;
	int ofx=0;
	if(tx) ofx=1;
	double start=0;
	if(lineCounter!=0) start=lineTimeTable[lineCounter-1];
	debugPtr->state=stColorLine0+colorLine;
	switch (colorLine)
		{
			case GREENLINE:
				start+=0;
			break;
			case BLUELINE:
				start+=(blank+visibleLineLength);
				syncEndPosition=(unsigned int)(start+visibleLineLength+fp+syncDuration);
//			debug("visible=%g,start=%u, syncend=%u",visibleLineLength,start,syncEndPosition);
			break;
			case REDLINE:
				start+=fp+syncDuration+bp+blank+2.*visibleLineLength;
			break;
		}
	for(i=0;i<activeSSTVParam.numberOfPixels;i++)
		{
			pixelPositionTable[i]=(unsigned int)round(start+(((float)(i+ofx)*visibleLineLength)/activeSSTVParam.numberOfPixels));
		}
}

embState gbr2Mode::txSetupLine(int &subln,unsigned char **buf)
{
//	debug("startImageTX");
	switch(subln)
		{
			case 0:
				calcPixelPositionTable(GREENLINE,TRUE);
				*buf=greenArray;
				return PIXELS;
			case 1:
				txFreq=1500.;
				txDur=blank;
				return TXGAP;
			case 2:
				calcPixelPositionTable(BLUELINE,TRUE);
				*buf=blueArray;
				return PIXELS;
			case 3:
				txFreq=1500;
				txDur=fp;
				return TXGAP;
			case 4:
				txFreq=1200;
				txDur=syncDuration;
				return TXGAP;
			case 5:
				txFreq=1500;
				txDur=bp;
				return TXGAP;	
			case 6:
				calcPixelPositionTable(REDLINE,TRUE);
				*buf=redArray;
				return PIXELS;
			case 7:
				txFreq=1500.;
				txDur=blank;
				return TXGAP;
			default:
				return ENDOFLINE;
		}
}
