/***************************************************************************
 *   Copyright (C) 2004 by Johan Maes ON4QZ                                *
 *   on4qz@telenet.be                                                      *
 *                                                                         *
 *   http://users.telenet.be/on4qz                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#ifndef LSOUNDCARD_H
#define LSOUNDCARD_H

#include <sys/soundcard.h>
#include <qobject.h>
#include <qstring.h>

#define AUDIOBUFFERSIZE 1024 //!< length in bytes of the audioBuffer
class QTimer;

//! structure describing the interface to /dev/dsp
struct dspstruct
  {
    int fdDSP;                      //!< filedescriptor for /dev/dsp
    unsigned int speed;             //!< sampling rate
    int channels;                   //!< 1=mono, 2=stereo
    int format;                     //!< AFMT_S16_LE (16-Bits) or AFMT_U8 (8-bits samples)
    unsigned int fragSize;          //!< Size of one fragment to process
    unsigned int numFrags;          //!< number of fragments
    unsigned int buffLen;           //!< length of the buffer
    unsigned int caps;              //!< Soundcard driver capabilities
    char * pbuffer;                 //!< DMA buffer. note: caddr_t = char *
  };

	
//! functions for soundcard support	
class soundcard:public QObject
{
	Q_OBJECT
 public:
  soundcard(int i_samplingRate=8000);
  ~soundcard();
  void setSamplingrate(int s);
  bool startReceive(const char *s,QString &errorString);
  bool startTransmit(const char *s,QString &errorString);
  bool startFullDuplex(const char *s,QString &errorString);
  void stop();
  bool read(char *audioBuffer);
	bool write(short int *longBuffer,uint length);
	bool write(char *audioBuffer);
	
  
  int waitEnd();
  
  int audioBufferLen; //!< length in bytes of the audioBuffer
  int fragmentLatency; //!< latency while reading/writing to soundcard 
 private:


  enum esoundStatus {CLOSED,OPENFORREAD,OPENFORWRITE,OPENFORREADWRITE};
  esoundStatus status;
  struct dspstruct dspInfo;

  bool writeWrapped;
  uint blockSize;
  uint writeIndex;
  uint writeLength;
  char *longBuffer;

public slots:
	 void delayedStop();

private slots: // Public slots
  //! write next audio block 
  void writeNext();
  //! wait for empty buffer 
  void slotStop();
signals:
	//! signal to indicate soundcard activity has stopped 
	void signalStopped();
	//! signal to indicate that there are  transmit buffers available
	void signalTransmitBufferAvailable();
private:
	bool setParam(int format, int fragsize,int channels,QString &errorString);
	bool started;
	bool busy;
	QTimer *timer1;
	QTimer *timer2; 	
};

#endif










